#include "SanguruSegmentBasic.h"
#include "Arduino.h"

// constructor 앞에는 void 조차도 없다.
SanguruSegmentBasic::SanguruSegmentBasic(byte inLatchPin, byte inClkPin, byte inDataPin) {
  latchPin = inLatchPin;
  clkPin = inClkPin;
  dataPin = inDataPin;  

  pinMode(latchPin,OUTPUT);
  pinMode(clkPin,OUTPUT);
  pinMode(dataPin,OUTPUT);
}
void SanguruSegmentBasic::writePattern(byte digit, byte pattern) {
  buffer[digit] = pattern;  
}
void SanguruSegmentBasic::write(int number) {
  write((float)number,0);
}

void SanguruSegmentBasic::write(float number, byte decimalPlaces) {
  int expo[4] = {1,10,100,1000};
  byte buffer[4] = {0,0,0,0};
  bool sign = false;
  
  if (decimalPlaces > 3 || decimalPlaces < 0)
    return;
    
  int val = number * expo[decimalPlaces];
  
  if (val > 9999 || val < -999) {
    writePattern(3,121); // 121=E
    return;
  }    
    
  if (val < 0 ) {
    val = 0 - val;
    sign = true;
  }
  int i;
  for (i = 3; i >= 0 && val != 0; val /= 10, i--) {
    buffer[i] = num[val % 10];
  }
  if (sign == true) {
      buffer[i] = 64; // minus(-)
  }
  if (decimalPlaces > 0) {
    buffer[3-decimalPlaces] += 128;   // decimal point
  }
 
  for (int i = 0; i <= 3; i++) {
    writePattern(i,buffer[i]);
  }
}
void SanguruSegmentBasic::update() {
  // 한 자릿수씩 번갈이 가면서 표시한다. 
  digitLoop++;
  if (digitLoop > 3)
    digitLoop = 0;

  // common digit의 배선 순서가 역순인 경우 대비 
  byte newDigitLoop;
  if (digitOrderLowOrHigh == LOW) 
  	newDigitLoop = digitLoop;
  else
    newDigitLoop = 3 - digitLoop; 
  
  // 공통 아노드 디스플레이의 경우 segment 낱개에 연결되는 핀이 LOW가 되어야 ON 상태가 된다.
  // 위 num[]은 사람이 이해하기 쉽게 핀이 HIGH가 되어야 ON 상태가 되도록 표시되어 있으므로 
  // shiftOut()하기 전에 buffer[]를 ~기호를 이용해서 반전시킨다.
  byte newDigit,newBuffer;
  if (commonAnode == true) {
  	newBuffer = ~buffer[newDigitLoop];
  	newDigit = digit[digitLoop];
  }
  else {
  	newBuffer = buffer[newDigitLoop];
  	newDigit = ~digit[digitLoop];
  }

  // 밝기를 1~7 단계로 조절 
  eachDigitLoop[digitLoop]++;

  if ( (bright == 1 && (eachDigitLoop[digitLoop] % 20 !=0)) ||
       (bright == 2 && (eachDigitLoop[digitLoop] % 13 !=0)) ||
       (bright == 3 && (eachDigitLoop[digitLoop] % 10 !=0)) ||
       (bright == 4 && (eachDigitLoop[digitLoop] % 6 !=0)) ||
       (bright == 5 && (eachDigitLoop[digitLoop] % 3 !=0)) ||
       (bright == 6 && (eachDigitLoop[digitLoop] % 2 !=0)) ) {
  	newBuffer = B00000000;
  	newDigit  = B00000000;
  }
  
  // LATCH핀이 LOW가 되면 74HC595의 레지스터에 비트를 받아들일 준비가 된다. 
  digitalWrite(latchPin,LOW); 
  shiftOut(dataPin, clkPin, MSBFIRST, newBuffer);
  shiftOut(dataPin, clkPin, MSBFIRST, newDigit );
  digitalWrite(latchPin,HIGH);
  // LATCH핀이 HIGH가 되면 74HC595의 레지스터에 있는 비트 값에 따라 해당 핀에 5V(1)나 0V(0)를 동시에 출력한다.
}
void SanguruSegmentBasic::setDigitOrderHigh() {
  digitOrderLowOrHigh = HIGH;
}
void SanguruSegmentBasic::setCommonCathode() {
  commonAnode = false;	
}
void SanguruSegmentBasic::setBrightness(byte inBright) {
  if (inBright > 7) 
  	inBright = 7;
  else if (inBright < 1) 
  	inBright = 1;
  bright = inBright;	
}